from typing import MutableMapping, Any, Text

_Attrs = MutableMapping[Any, Text]

def nofollow(attrs: _Attrs, new: bool = ...) -> _Attrs: ...
def target_blank(attrs: _Attrs, new: bool = ...) -> _Attrs: ...
